import omise
from django.conf import settings

from typing import Optional, Any


def setting(name: str, default: Optional[Any] = None) -> Optional[Any]:
    """
    Helper function to get a Django setting by name. If setting doesn't exists
    it will return a default.

    :param name: Name of setting
    :param default: Value if setting is unfound
    :returns: Setting's value
    """
    return getattr(settings, name, default)


def is_omise_object_instances(value) -> bool:
    """
    Check if a value is an instance of Omise Classes.

    See all the classes here https://github.com/omise/omise-python/blob/master/omise/__init__.py

    :param value: The value to check.

    :returns: True if value is an instance of Omise Classes, False otherwise
    """
    return isinstance(
        value,
        (
            omise.Account,
            omise.Balance,
            omise.BankAccount,
            omise.Capability,
            omise.Card,
            omise.Chain,
            omise.Charge,
            omise.Collection,
            omise.Customer,
            omise.Dispute,
            omise.Document,
            omise.Event,
            omise.Forex,
            omise.Link,
            omise.Occurrence,
            omise.Receipt,
            omise.Recipient,
            omise.Refund,
            omise.Search,
            omise.Schedule,
            omise.Source,
            omise.Token,
            omise.Transaction,
            omise.Transfer,
        ),
    )



from .forms import CheckoutWithCardsForm
from .models.core import Customer, Charge, Card
from .models.choices import Currency, ChargeStatus
from .omise import omise

from django.contrib import messages
from django.shortcuts import redirect
from django.utils.translation import gettext as _
from django.views.generic import FormView

from typing import Dict


class CheckoutWithCardsMixin(FormView):
    form_class = CheckoutWithCardsForm

    def charge_with_card(self, card: Card) -> Charge:
        """
        Charge the user with selected card.

        :returns: An instance of Charge object
        """
        customer = self.get_omise_customer()
        charge_details = self.get_charge_details()
        charge = customer.charge_with_card(
            amount=charge_details["amount"],
            currency=charge_details["currency"],
            card=card,
            **self.get_charge_kwargs(),
        )

        return charge

    def get_charge_details(self) -> Dict[int, Currency]:
        """
        Overwrite this method to provide the charge amount and currency.

        Read more on the amount and currency here https://www.omise.co/currency-and-amount

        :returns: Dictionary of charge amount in the smallest unit (int) and the 3-digit currency code. {"amount": int, "currency": str}
        """

    def get_omise_customer(self) -> Customer:
        """
        Overwrite this method to provide the customer instance to charge.

        :returns: An instance of Customer
        """
        if self.request.user.is_authenticated:
            customer, created = Customer.get_or_create(user=self.request.user)
            return customer

    def get_charge_kwargs(self) -> Dict:
        """
        Overwrite this method to add charge parameters.

        :return: A dictionary of charge parameters
        """
        return dict()

    def form_valid(self, form):

        selected_card = form.cleaned_data["card"]

        try:
            charge = self.charge_with_card(card=selected_card)
        except omise.errors.InvalidChargeError as e:
            messages.error(
                self.request,
                f"Payment failed: {str(e)}",
            )
            return super().form_invalid(form)

        self.process_new_charge(charge)

        if charge.status == ChargeStatus.FAILED:
            messages.error(
                self.request,
                f"Payment failed: {charge.failure_message}",
            )
            return super().form_invalid(form)

        if charge.status == ChargeStatus.PENDING:
            return redirect(charge.authorize_uri)

        if charge.status == ChargeStatus.SUCCESSFUL:
            messages.success(self.request, _("Payment successful."))

        return super().form_valid(form)

    def process_new_charge(self, charge: Charge):
        """
        Overwrite this method to process the new charge object.

        :params charge: The new Charge object.
        """
        pass



import uuid
from django.db import models


class OmiseBaseModel(models.Model):

    id = models.CharField(max_length=255, primary_key=True)
    livemode = models.BooleanField()
    data = models.JSONField(default=dict, blank=True)

    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    uid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)

    NON_DEFAULT_FIELDS = [
        "id",
        "livemode",
        "date_created",
        "date_updated",
        "data",
        "uid",
    ]

    def __str__(self):
        return f"Omise{self.__class__.__name__}: {self.id}"

    class Meta:
        abstract = True
        ordering = [
            "-date_created",
        ]



from django.urls import path, include

from .views import (
    omise_webhook_view,
    ManagePaymentMethodsView,
    PaymentMethodDeleteView,
    OmiseReturnURIView,
)

app_name = "django_omise"

urlpatterns = [
    path("return/<uuid:uid>/", OmiseReturnURIView.as_view(), name="return_uri"),
    path("webhook/", omise_webhook_view, name="webhook"),
    path(
        "payment_methods/",
        include(
            [
                path(
                    "",
                    ManagePaymentMethodsView.as_view(),
                    name="manage_payment_methods",
                ),
                path(
                    "delete/<pk>/",
                    PaymentMethodDeleteView.as_view(),
                    name="delete_payment_method",
                ),
            ]
        ),
    ),
]



from django import forms
from django.conf import settings

from .omise import omise

from django.utils.translation import gettext_lazy as _

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .models.core import Customer


class AddCardForm(forms.Form):

    omise_token = forms.CharField(
        widget=forms.HiddenInput,
    )

    omise_public_key = forms.CharField(
        widget=forms.HiddenInput,
        disabled=True,
        initial=settings.OMISE_PUBLIC_KEY,
    )

    card_number = forms.CharField(
        required=False,
    )

    name_on_card = forms.CharField(
        required=False,
    )

    expiration_month = forms.CharField(
        required=False,
    )

    expiration_year = forms.CharField(
        required=False,
    )

    security_number = forms.CharField(
        required=False,
    )

    class Media:
        js = (
            "django_omise/js/jquery.min.js",
            "django_omise/js/jquery.payments.js",
            "django_omise/js/omise.js",
            "django_omise/js/add_card.js",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["card_number"].widget.attrs["placeholder"] = "1234 1234 1234 1234"
        self.fields["name_on_card"].widget.attrs["placeholder"] = "John Doe"
        self.fields["expiration_month"].widget.attrs["placeholder"] = "1-12"
        self.fields["expiration_year"].widget.attrs["placeholder"] = "2030"
        self.fields["security_number"].widget.attrs["placeholder"] = "***"

    def clean_omise_token(self) -> omise.Token:
        """
        Check that the credit card token is valid.
        """
        try:
            omise_token = omise.Token.retrieve(self.cleaned_data["omise_token"])
        except omise.errors.NotFoundError:
            raise forms.ValidationError(
                _("Error processing the card. Please try again.")
            )

        return omise_token

    def add_card_to_user(self, customer: "Customer"):
        """
        Add the new card to the user.

        :params customer: The customer instance

        :returns None:
        """
        customer.add_card(self.cleaned_data["omise_token"])


class CheckoutWithCardsForm(forms.Form):

    card = forms.ModelChoiceField(queryset=None, widget=forms.RadioSelect)

    def __init__(self, user, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.user = user
        self.omise_customer = self.user.extension.omise_customer

        self.fields["card"].queryset = self.omise_customer.cards.live()
